import matplotlib.pyplot as plt
import numpy as np
from oving4.lattice import Pos, Lattice, Transition
import random

rows = 20
cols = 20
N = 100
T = 50 #when using interaction energies
num_steps = 3000
dump_interval = 250

def initialize_left():
    positions = [None for i in range(N)]
    cols_to_fill = N // rows
    last_col_rows = N % rows

    if cols_to_fill == 0:
        i = 0
        for r in range(N):
            positions[i] = Pos(r, 0)

    else:
        i = 0
        for c in range(cols_to_fill):
            for r in range(rows):
                positions[i] = Pos(r, c)
                i += 1

        for r in range(last_col_rows):
            positions[i] = Pos(r, cols_to_fill)
            i += 1

    return positions

def valid_move(move, positions):
    if move in positions:
        return False
    elif move.row < 0 or move.row >= rows:
        return False
    elif move.col < 0 or move.col >= cols:
        return False
    return True

def possible_transitions(positions):
    transitions = []

    moves = [Pos(-1, 0), Pos(1, 0), Pos(0, -1), Pos(0, 1)]
    for i, p in enumerate(positions):
        for m in moves:
            if valid_move(p+m, positions):
                transitions.append(Transition(p, m, positions))

    return transitions

def perform_transition_energy(positions, transitions):
    energies = np.array([t.energy for t in transitions])
    factors = np.exp(-energies / T)
    probs = factors/sum(factors)

    intervals = [sum(probs[:i]) for i in range(len(probs)+1)]
    ran = np.random.rand()
    for i in range(len(intervals)-1):
        if intervals[i] < ran < intervals[i+1]:
            trans_ind = i
            break

    for i in range(len(positions)):
        positions[i] += transitions[trans_ind]
    return positions

def perform_transition(positions, transitions):
    trans_ind = random.randint(0, len(transitions)-1)
    for i in range(len(positions)):
        positions[i] += transitions[trans_ind]
    return positions

def save_image(lattice, TrNum): #TrNum keeps track of the step number in the loop.
    imgplot = plt.imshow(lattice.numlat, cmap='binary')
    plt.show()
    #plt.savefig('figs/Lattice' + str(TrNum) + '.png')

pos = initialize_left()
t_list = [t for t in range(num_steps)]
entropy = np.zeros(len(t_list))
lattice = Lattice(rows, cols, pos)
for i in range(num_steps):
    trans = possible_transitions(pos)

    # energies = np.array([t.energy for t in trans])
    # factors = np.exp(-energies / T)
    # probs = factors / sum(factors)
    #
    # print(lattice)
    # for trs, p in zip(trans, probs):
    #     print(trs, ', p =',round(p, 3))

    pos = perform_transition_energy(pos, trans)
    lattice = Lattice(rows, cols, pos)
    entropy[i] = lattice.get_entropy()

    if i%dump_interval == 0:
        print(i)
        save_image(lattice, i)

#Use the initialize_left function to create a list of all the particles positions
#Set up the list with possible transitions
#Remember to set up lists for the time stamp and the local entropy that can be updated inside the loop

#Start the loop. Both a for-loop and a while-loop will work.
    #Update the positions lists using Perform_Transition()
    #Calculate the present time: time += "KMC equation for time". Note that the time is updated in every step but only stored when the entropy is stored
    #Recalculate the possible transitions (too complex to update, just recalculate from scratch
        #Remember to store an image at a regular interval. Use for example: if TrNum % Dump_interval == 0:
        #Update the entropy and time stamp lists, also at a regular interval. Make sure that the time step list reflects the total time passed at any given point.
    #If using a while-loop, update TrNum

#The code below creates the plots the local entropy as a function of time.
plt.clf()
plt.plot(t_list, entropy)
plt.show()