import numpy as np
from copy import deepcopy
from math import log, factorial

GRAVITY = False
DENSITY_DIFFERENCE = 100
SITE_INTERACT_ENERGY = 0
SELF_INTERACT_ENERGY = 0

class Pos:
    def __init__(self, row, col):
        self.row = row
        self.col = col
        self.pos = (row, col)

    def __add__(self, other):
        if type(other) == type(self):
            row = self.row + other.row
            col = self.col + other.col
            return Pos(row, col)

        elif type(other) == Transition:
            if self == other.pos_from:
                return Pos(other.pos_to.row, other.pos_to.col)
            else:
                return Pos(self.row, self.col)

    __radd__ = __add__

    def __eq__(self, other):
        if self.row == other.row and self.col == other.col:
            return True
        return False

    def __str__(self):
        return str(self.pos)

class Transition:
    def __init__(self, pos_from, move, positions):
        self.pos_from = deepcopy(pos_from)
        self.pos_to = pos_from + move
        self.move = deepcopy(move)

        self.energy = 0
        check_pos = [Pos(-1, 0), Pos(1, 0), Pos(0, -1), Pos(0, 1)]
        for p in check_pos:
            if self.pos_to + p in positions:
                if self.pos_to + p == self.pos_from:
                    self.energy += SITE_INTERACT_ENERGY
                else:
                    self.energy += SELF_INTERACT_ENERGY
            else:
                self.energy += SITE_INTERACT_ENERGY

        if GRAVITY is True:
            self.energy += DENSITY_DIFFERENCE * self.move.row

    def __str__(self):
        return str(self.pos_from) + ' -> ' + str(self.pos_to) + ', E = '+str(round(self.energy, 2))

class Lattice:
    def __init__(self, nrows, ncols, positions):
        self.nrows = nrows
        self.ncols = ncols
        self.strlat = [[' ' for c in range(ncols)] for r in range(nrows)]
        self.numlat = [[0 for c in range(ncols)] for r in range(nrows)]
        self.positions = deepcopy(positions)

        self.npoints = nrows * ncols
        self.N = len(self.positions)

        for p in positions:
            self.strlat[p.row][p.col] = 'x'
            self.numlat[p.row][p.col] = 1

    def get_local_lattice(self):
        ncols = max([p.col for p in self.positions]) + 1
        nrows = max([p.row for p in self.positions]) + 1

        return Lattice(nrows, ncols, self.positions)

    def get_entropy(self):
        local_lattice = self.get_local_lattice()
        M = local_lattice.npoints
        return log(factorial(M) / (factorial(self.N) * factorial(M - self.N)))

    def __str__(self):
        ret_str = '   '
        for i in range(self.ncols):
            ret_str += ' ' + str(i) +' '
        ret_str += '\n'
        ret_str += '   ' + ' - '*self.ncols + '|\n'
        for i, row in enumerate(self.strlat):
            ret_str += str(i)+ ' |'
            for val in row:
                ret_str += ' '+val+' '
            ret_str += '|\n'

        ret_str += '  |' + ' - ' * self.ncols + '|'

        return ret_str
